@@ -25,6 +25,7 @@ changes and checking if the data is still there.
 import time
 
 import os
+import dbus # Just for the Exception
 from common.utils import configuration as cfg
 import unittest2 as ut
 #import unittest as ut
@@ -174,7 +175,6 @@ class PropertyRangeStringToDate (OntologyChangeTestTemplate):
     Change the range of a property from string to date. There shouldn't be any data loss.
     """
 
-    @expectedFailureBug ("New journal is gonna work it out")
     def test_property_range_string_to_date (self):
         self.template_test_ontology_change ()
 
@@ -203,7 +203,6 @@ class PropertyRangeDateToString (OntologyChangeTestTemplate):
     Change the range of a property from date to string. There shouldn't be any data loss.
     """
 
-    @expectedFailureBug ("New journal is gonna work it out")
     def test_property_range_date_to_string (self):
         self.template_test_ontology_change ()
 
@@ -213,16 +212,16 @@ class PropertyRangeDateToString (OntologyChangeTestTemplate):
         
     def insert_data (self):
         self.instance = "test://ontology-change/property-range/date-to-string"
-        self.tracker.update ("INSERT { <%s> a test:classA ; test:propertyString '2010-10-12T13:30:00Z' }"
+        self.tracker.update ("INSERT { <%s> a test:A ; test:a_string '2010-10-12T13:30:00Z' }"
                              % (self.instance))
 
     def validate_status (self):
         # Query the ontology itself
-        result = self.tracker.query ("SELECT ?o WHERE { test:propertyString rdfs:range ?o }")
+        result = self.tracker.query ("SELECT ?o WHERE { test:a_string rdfs:range ?o }")
         self.assertEquals (result[0][0], XSD_STRING)
 
         # Check the value is there
-        result = self.tracker.query ("SELECT ?o WHERE { <%s> test:propertyString ?o . }" % (self.instance))
+        result = self.tracker.query ("SELECT ?o WHERE { <%s> test:a_string ?o . }" % (self.instance))
         self.assertEquals (result[0][0], "2010-10-12T13:30:00Z")
 
 class PropertyRangeIntToString (OntologyChangeTestTemplate):
@@ -277,7 +276,6 @@ class PropertyMaxCardinality1toN (OntologyChangeTestTemplate):
     Change cardinality of a property from 1 to N. There shouldn't be any data loss
     """
 
-    @expectedFailureBug ("New journal is gonna work it out")
     def test_property_cardinality_1_to_n (self):
         self.template_test_ontology_change ()
 
@@ -309,7 +307,6 @@ class PropertyMaxCardinalityNto1 (OntologyChangeTestTemplate):
     Change the cardinality of a property for N to 1.
     """
 
-    @expectedFailureBug ("New journal is gonna work it out")
     def test_property_cardinality_n_to_1 (self):
         self.template_test_ontology_change ()
 
@@ -608,45 +605,6 @@ class OntologyRemovePropertyTest (OntologyChangeTestTemplate):
         check = self.tracker.ask ("ASK { <%s> test:a_superprop_n 'super-prop also keeps this value' }" % (self.instance_b))
         self.assertTrue (check, "This property and value should exist")
 
-class DomainIndexAddTest (OntologyChangeTestTemplate):
-    """
-    Add tracker:domainIndex in properties
-    """
-    def test_DomainIndexAddTest (self):
-        self.template_test_ontology_change ()
-
-    def set_ontology_dirs (self):
-        self.FIRST_ONTOLOGY_DIR = "basic"
-        self.SECOND_ONTOLOGY_DIR = "add-domainIndex"
-
-    def insert_data (self):
-        self.instance_a = "test://ontology-changes/properties/add-domain-index/a"
-        self.tracker.update ("""
-            INSERT { <%s> a test:B ;
-                          test:a_string 'test-value' ;
-                          test:a_n_cardinality 'another-test-value'. }""" % (self.instance_a))
-
-        self.instance_b = "test://ontology-changes/properties/add-domain-index/b"
-        self.tracker.update ("""
-            INSERT { <%s> a test:C ;
-                          test:a_string 'test-value' ;
-                          test:a_n_cardinality 'another-test-value'. }""" % (self.instance_b))
-
-    def validate_status (self):
-        # Check the ontology
-        has_domainIndex = self.tracker.ask ("ASK { test:B tracker:domainIndex test:a_string }")
-        self.assertTrue (has_domainIndex)
-
-        has_domainIndex = self.tracker.ask ("ASK { test:C tracker:domainIndex test:a_n_cardinality }")
-        self.assertTrue (has_domainIndex)
-
-        # Check the data
-        dataok = self.tracker.ask ("ASK { <%s> test:a_string 'test-value' }" % (self.instance_a))
-        self.assertTrue (dataok)
-
-        dataok = self.tracker.ask ("ASK { <%s> test:a_n_cardinality 'another-test-value' }" % (self.instance_b))
-        self.assertTrue (dataok)
-
 class DomainIndexAddTest (OntologyChangeTestTemplate):
     """
     Add tracker:domainIndex to a class and check there is no data loss.
@@ -727,6 +685,158 @@ class DomainIndexRemoveTest (OntologyChangeTestTemplate):
         self.assertTrue (dataok)
 
 
+class SuperclassRemovalTest (OntologyChangeTestTemplate):
+    """
+    Remove the superclass relation between two classes
+    """
+    def test_superclass_removal (self):
+        self.template_test_ontology_change ()
+        
+    def set_ontology_dirs (self):
+        self.FIRST_ONTOLOGY_DIR = "basic"
+        self.SECOND_ONTOLOGY_DIR = "superclass-remove"
+            
+    def insert_data (self):
+        is_subclass = self.tracker.ask ("ASK {test:B rdfs:subClassOf test:A}")
+        self.assertTrue (is_subclass)
+        
+        self.instance_a = "test://ontology-changes/superclasses/remove-superclass/a"
+        self.tracker.update ("""
+         INSERT { <%s> a test:A . }
+        """ % (self.instance_a))
+
+        self.instance_b = "test://ontology-changes/superclasses/remove-superclass/b"
+        self.tracker.update ("""
+         INSERT { <%s> a test:B . }
+        """ % (self.instance_b))
+
+        result = self.tracker.count_instances ("test:B")
+        self.assertEquals (result, 1)
+
+        result = self.tracker.count_instances ("test:A")
+        self.assertEquals (result, 2)
+
+    def validate_status (self):
+        is_subclass = self.tracker.ask ("ASK {test:B rdfs:subClassOf test:A}")
+        self.assertFalse (is_subclass)
+
+        result = self.tracker.count_instances ("test:B")
+        self.assertEquals (result, 1)
+
+        result = self.tracker.count_instances ("test:A")
+        self.assertEquals (result, 1)
+
+class SuperclassAdditionTest (OntologyChangeTestTemplate):
+    """
+    Add a superclass to a class with no superclass previously
+    """
+    def test_superclass_addition (self):
+        self.template_test_ontology_change ()
+        
+    def set_ontology_dirs (self):
+        self.FIRST_ONTOLOGY_DIR = "superclass-remove"
+        self.SECOND_ONTOLOGY_DIR = "basic-future"
+            
+    def insert_data (self):
+        is_subclass = self.tracker.ask ("ASK {test:B rdfs:subClassOf test:A}")
+        self.assertFalse (is_subclass)
+        
+        self.instance_a = "test://ontology-changes/superclasses/remove-superclass/a"
+        self.tracker.update ("""
+         INSERT { <%s> a test:A . }
+        """ % (self.instance_a))
+
+        self.instance_b = "test://ontology-changes/superclasses/remove-superclass/b"
+        self.tracker.update ("""
+         INSERT { <%s> a test:B . }
+        """ % (self.instance_b))
+
+        result = self.tracker.count_instances ("test:B")
+        self.assertEquals (result, 1)
+
+        result = self.tracker.count_instances ("test:A")
+        self.assertEquals (result, 1)
+        
+    def validate_status (self):
+        is_subclass = self.tracker.ask ("ASK {test:B rdfs:subClassOf test:A}")
+        self.assertTrue (is_subclass)
+
+        result = self.tracker.count_instances ("test:B")
+        self.assertEquals (result, 1)
+
+        result = self.tracker.count_instances ("test:A")
+        self.assertEquals (result, 2)
+        
+
+class PropertyPromotionTest (OntologyChangeTestTemplate):
+    """
+    Move a property to the superclass
+    """
+    def test_property_promotion (self):
+        self.template_test_ontology_change ()
+        
+    def set_ontology_dirs (self):
+        self.FIRST_ONTOLOGY_DIR = "basic"
+        self.SECOND_ONTOLOGY_DIR = "property-promotion"
+            
+    def insert_data (self):
+        self.instance_b = "test://ontology-change/property/promotion-to-superclass/b"
+        self.tracker.update ("""
+            INSERT { <%s> a test:B; test:b_property 'content-b-test'; test:b_property_n 'b-test-n'. }
+           """ % (self.instance_b))
+
+        self.instance_a = "test://ontology-change/property/promotion-to-superclass/a"
+        self.assertRaises (dbus.DBusException,
+                           self.tracker.update,
+                           "INSERT { <%s> a test:A; test:b_property 'content-a-test'.}" % (self.instance_a))
+        
+    def validate_status (self):
+        # This insertion should work now
+        self.tracker.update ("""
+           INSERT { <%s> a test:A; test:b_property 'content-a-test'.}
+        """ % (self.instance_a))
+
+        # No data loss
+        result = self.tracker.query ("SELECT ?v ?w WHERE { <%s> test:b_property ?v ; test:b_property_n ?w }"
+                                     % (self.instance_b))
+        self.assertEquals (result [0][0], "content-b-test")
+        self.assertEquals (result [0][1], "b-test-n")
+
+class PropertyRelegationTest (OntologyChangeTestTemplate):
+    """
+    Move a property to the subclass
+    """
+    def test_property_relegation (self):
+        self.template_test_ontology_change ()
+        
+    def set_ontology_dirs (self):
+        self.FIRST_ONTOLOGY_DIR = "property-promotion"
+        self.SECOND_ONTOLOGY_DIR = "basic-future"
+            
+    def insert_data (self):
+        self.instance_b = "test://ontology-change/property/promotion-to-superclass/b"
+        self.tracker.update ("""
+            INSERT { <%s> a test:B; test:b_property 'content-b-test'; test:b_property_n 'b-test-n'. }
+           """ % (self.instance_b))
+
+        self.instance_a = "test://ontology-change/property/promotion-to-superclass/a"
+        self.tracker.update ("""
+           INSERT { <%s> a test:A; test:b_property 'content-a-test'.}
+        """ % (self.instance_a))
+        
+    def validate_status (self):
+        # This insertion should fail now
+        self.assertRaises (dbus.DBusException,
+                           self.tracker.update,
+                           "INSERT { <%s> a test:A; test:b_property 'content-a-test'.}" % (self.instance_a))
+        # No data loss
+        result = self.tracker.query ("SELECT ?v ?w WHERE { <%s> test:b_property ?v; test:b_property_n ?w }"
+                                     % (self.instance_b))
+        self.assertEquals (result [0][0], "content-b-test")
+        self.assertEquals (result [0][1], "b-test-n")
+
+
+
 if __name__ == "__main__":
     ut.main ()
 
